/*
 Copyright (c) 2010 Myles Metzer

 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
 files (the "Software"), to deal in the Software without
 restriction, including without limitation the rights to use,
 copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the
 Software is furnished to do so, subject to the following
 conditions:

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
*/

/*
 This library provides a simple method for outputting data to a tv
 from a frame buffer stored in sram.  This implementation is done
 completly by interupt and will return give as much cpu time to the
 application as possible.
 
 Limitations:
  -	Currently only works with NTSC. PAl support would require modifying
	the timings in video_gen.h as well as the OCR1A ISR to allow for a 16bit
	line counter.
  -	each frame only consists of 256 scanlines vs 262 for the fake progressive
	signal this is supposed to generate (works fine on my tv).
  - virtical sync does not match the specs at all.
 

 current hardware setup:
 Pin9: Sync line:  -->|--/\/\/\--o
                   Diode 1Kohm   |
 Pin8: Video line: -->|--/\/\/\--o---------------- RCA tip
                   Diode 330ohm  |
                                 o--/--/\/\/\--o-- RCA GND
							     Switch  75ohm  |
									            V
											   GND
											   
 
*/


/**/


#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <stdlib.h>

#include <math.h>
#include <stdio.h>

#include "TVoutTBF.h"

#include "video_gen.h"

#include "video_properties.h"

PROGMEM const unsigned char ascii[] = { 
#include "font_set.h"
};

void TVout::scan_line(unsigned int v){
	scan_lines = v;
}

/* call this to start video output
*/
void TVout::start_render(unsigned char mode) {

	g_scanLine = 0;
	clear_screen();
	
#if defined(__AVR_ATmega1280__)
	DDRB |= 0x60;
	PORTB &= ~0x60;
	PORTB |= 0x20;
#else
	//setup the ports
	DDRB |= 0x03;
	PORTB &= ~0x02;
	PORTB |= 0x02;
#endif
	
	// inverted fast pwm mode on timer 1
	TCCR1A = _BV(COM1A1) | _BV(COM1A0) | _BV(WGM11);
	TCCR1B = _BV(WGM13) | _BV(WGM12) | _BV(CS10);
	
	if (mode) {
		g_status = _VID_MODE;
		ICR1 = _PAL_CYCLES_SCANLINE;
		OCR1A = _PAL_CYCLES_HORZ_SYNC;
		}
	else {
		g_status = 0;
		ICR1 = _NTSC_CYCLES_SCANLINE;
		OCR1A = _NTSC_CYCLES_HORZ_SYNC;
	}

	TIMSK1 = _BV(OCIE1B);
	
	sei();
}

/* pauses rendering but keeps outputting a HSYNC this gives all CPU time to
 * user code
*/
void TVout::pause_render() {
	TIMSK1 = 0;
	if (_IS_NTSC) {
		ICR1 = _NTSC_CYCLES_SCANLINE;
		OCR1A = _NTSC_CYCLES_HORZ_SYNC;
	}
	else {
		ICR1 = _PAL_CYCLES_SCANLINE;
		OCR1A = _PAL_CYCLES_HORZ_SYNC;
	}
}

/* Resume rendering the screen
*/
void TVout::resume_render() {
	g_scanLine = 0;
	g_status &= ~_RENDER;
	TIMSK1 = _BV(OCIE1B);
}

/* Clears the screen
*/
void TVout::clear_screen() {
	for (int i = 0; i < (_RESOLUTION_HORZ_BYTES)*_RESOLUTION_VERTICAL; i++)
		g_screen[i] = 0;
}

/* Gets the Horizontal resolution of the screen
*/
unsigned char TVout::horz_res() {
	return _RESOLUTION_HORIZONTAL-1;
}

/* Gets the Vertical resolution of the screen
*/
unsigned char TVout::vert_res() {
	return _RESOLUTION_VERTICAL;
}

/* Return the number of characters that will fit on a line
*/
char TVout::char_line() {
	return ((_RESOLUTION_HORIZONTAL/6)-1);
}

/* Delay for x frames
 * for NTSC 1 second = 60 frames
 * for PAL 1 second = 50 frames
 */
void TVout::delay_frame(unsigned int x) {
	if (_IS_NTSC) {
		while (x > 0) {
			while (g_scanLine != _NTSC_LINE_STOP_RENDER);
			while (g_scanLine == _NTSC_LINE_STOP_RENDER);
			x--;
		}
	}
	else {
		while (x > 0) {
			while (g_scanLine != _PAL_LINE_STOP_RENDER);
			while (g_scanLine == _PAL_LINE_STOP_RENDER);
			x--;
		}
	}
}

/* plot one point 
 * at x,y with color 1=white 0=black 2=invert
 */
void TVout::set_pixel(unsigned char x, unsigned char y, char c) {
	if (x >= _RESOLUTION_HORIZONTAL || y >= _RESOLUTION_VERTICAL)
		return;
	sp(x,y,c);
}

/* Returns the value of pixel (x,y)
 * 0 if pixel is black
 * !0 if pixel is white
*/
unsigned char TVout::get_pixel(unsigned char x, unsigned char y, unsigned char m) {
	if (x >= _RESOLUTION_HORIZONTAL || y >= _RESOLUTION_VERTICAL)
		return 0;
	if(m)
		return ((g_screen[(x/8)+(y*_RESOLUTION_HORZ_BYTES)] >> ((~x) & 7)) &0x01);
	else
		return ((g_screen[(x/8)+(y*_RESOLUTION_HORZ_BYTES)] >> (x & 7)) &0x01);
}

/* draw a line
 * x1,y1 to x2,y2
 * with color 1 = white, 0=black, 2=invert
 */

void TVout::draw_line(unsigned char x0, unsigned char y0,
					  unsigned char x1, unsigned char y1, char c) {
	if (x0 >= _RESOLUTION_HORIZONTAL || y0 >= _RESOLUTION_VERTICAL ||
		x1 >= _RESOLUTION_HORIZONTAL || y1 >= _RESOLUTION_VERTICAL)
		return;
		
	//if (x0 > g_hres*8 || y0 > g_vres || x1 > g_hres*8 || y1 > g_vres)
	//	return;
	int e;
	signed int dx,dy,j, temp;
	signed char s1,s2, xchange;
    signed int x,y;
        
	x = x0;
	y = y0;
	
	//take absolute value
	if (x1 < x0) {
		dx = x0 - x1;
		s1 = -1;
	}
	else if (x1 == x0) {
		dx = 0;
		s1 = 0;
	}
	else {
		dx = x1 - x0;
		s1 = 1;
	}

	if (y1 < y0) {
		dy = y0 - y1;
		s2 = -1;
	}
	else if (y1 == y0) {
		dy = 0;
		s2 = 0;
	}
	else {
		dy = y1 - y0;
		s2 = 1;
	}

	xchange = 0;   

	if (dy>dx) {
		temp = dx;
		dx = dy;
		dy = temp;
		xchange = 1;
	} 

	e = ((int)dy<<1) - dx;  
	 
	for (j=0; j<=dx; j++) {
		/*if(gp(x,y)){
			break;
		}*/
		sp(x,y,c);
		 
		if (e>=0) {
			if (xchange==1) x = x + s1;
			else y = y + s2;
			e = e - ((int)dx<<1);
		}
		if (xchange==1)
			y = y + s2;
		else
			x = x + s1;
		e = e + ((int)dy<<1);
	}
}

void TVout::draw_line_limited(unsigned char x0, unsigned char y0,
					  unsigned char x1, unsigned char y1, char c) {
	if (x0 >= _RESOLUTION_HORIZONTAL || y0 >= _RESOLUTION_VERTICAL ||
		x1 >= _RESOLUTION_HORIZONTAL || y1 >= _RESOLUTION_VERTICAL)
		return;
		
	//if (x0 > g_hres*8 || y0 > g_vres || x1 > g_hres*8 || y1 > g_vres)
	//	return;
	int e;
	signed int dx,dy,j, temp;
	signed char s1,s2, xchange;
    signed int x,y;
        
	x = x0;
	y = y0;
	
	//take absolute value
	if (x1 < x0) {
		dx = x0 - x1;
		s1 = -1;
	}
	else if (x1 == x0) {
		dx = 0;
		s1 = 0;
	}
	else {
		dx = x1 - x0;
		s1 = 1;
	}

	if (y1 < y0) {
		dy = y0 - y1;
		s2 = -1;
	}
	else if (y1 == y0) {
		dy = 0;
		s2 = 0;
	}
	else {
		dy = y1 - y0;
		s2 = 1;
	}

	xchange = 0;   

	if (dy>dx) {
		temp = dx;
		dx = dy;
		dy = temp;
		xchange = 1;
	} 

	e = ((int)dy<<1) - dx;  
	 
	for (j=0; j<=dx; j++) {
		if(gp(x,y)){
			break;
		}
		sp(x,y,c);
		
		if (e>=0) {
			if (xchange==1) x = x + s1;
			else y = y + s2;
			e = e - ((int)dx<<1);
		}
		if (xchange==1)
			y = y + s2;
		else
			x = x + s1;
		e = e + ((int)dy<<1);
	}
}

/* print a character c at x,y
*/
void TVout::print_char(unsigned char x, unsigned char y, char c) { 
    char i;
	char y_pos;
	uint8_t j;

	for (i=0;i<7;i++) {
        y_pos = y + i;

		j = pgm_read_byte(((uint32_t)(ascii)) + c*7 + i);

        sp(x,   y_pos, (j & 0x80)==0x80);  
        sp(x+1, y_pos, (j & 0x40)==0x40); 
        sp(x+2, y_pos, (j & 0x20)==0x20);
        sp(x+3, y_pos, (j & 0x10)==0x10);
        sp(x+4, y_pos, (j & 0x08)==0x08);
    }
}

/* print a null terminated string at x,y
*/
void TVout::print_str(unsigned char x, unsigned char y, char *str) {
	if (x >= _RESOLUTION_HORIZONTAL || y >= _RESOLUTION_VERTICAL)
		return;
	for (char i=0; str[i]!=0; i++) { 
		print_char(x,y,str[i]);
		x += 6;
		if(x/6 >= char_line()) {
			x=0;
			y+=8;
		}
	}
}

void TVout::invert() {
	for (int i = 0; i < (_RESOLUTION_HORZ_BYTES)*_RESOLUTION_VERTICAL; i++)
		g_screen[i] = ~g_screen[i];
}

/*
void TVout::flood_fill(int x, int y, unsigned char c){
	if (x >= _RESOLUTION_HORIZONTAL || y >= _RESOLUTION_VERTICAL || x <= 0 || y <= 0)
		return;
		
	if (gp(x,y) == 0){
		sp(x,y,1);
		//flood_fill(x+1,y,1);
		flood_fill(x-1,y,1);
		flood_fill(x,y+1,1);
		//flood_fill(x,y-1,1);
	}
}

*/

void TVout::flood_fill(int x, int y, unsigned char c){
	if (x >= _RESOLUTION_HORIZONTAL || y >= _RESOLUTION_VERTICAL || x <= 0 || y <= 0)
		return;
		char x1 = 0;
		char y1 = 0;
		
		for (;x1 < _RESOLUTION_HORIZONTAL-1;x1++) {
			draw_line_limited(x,y,x1,y1,1);
  		}
  		for (;y1 < _RESOLUTION_VERTICAL-1;y1++) {
    		draw_line_limited(x,y,x1,y1,1);
  		}
  		for (;x1 > 0;x1--) {
    		draw_line_limited(x,y,x1,y1,1);
 		}
		for (;y1 > 0;y1--) {
 		    draw_line_limited(x,48,x1,y1,1);
	    }

}

void TVout::shift(unsigned char c, char speed) {
	unsigned int length = (_RESOLUTION_HORZ_BYTES)*_RESOLUTION_VERTICAL;
	unsigned int count = 0;
	char s1 = speed;
	char s2 = 8-speed;
	
	if(c == 0){
		for (int i = 1; i < length; i++){
			g_screen[i] = (g_screen[i] >> s1)|g_screen[count] << s2;
			count++;
			if(count >= length){
				count = 0;
			}
		}
	}else{
		for (int i = 0; i < length; i++){
			count++;
			if(count >= length){
				count = 0;
			}
			g_screen[i] = (g_screen[i] << s1)|g_screen[count] >> s2;
		}
	}
}

void TVout::scroll(unsigned char x0, unsigned char y0, unsigned char x1, unsigned char y1, char c) {

	if (x0 >= _RESOLUTION_HORIZONTAL || y0 >= _RESOLUTION_VERTICAL || x1 >= _RESOLUTION_HORIZONTAL || y1 >= _RESOLUTION_VERTICAL){
		return;
	}
		
	if(c==1){
		//scroll up
		for(char j = y0; j < y1; j++){
			for(char i = x0; i < x1; i++){
				char ty = j+1;
				if(ty >= y1){
					ty = y0;
				}
				sp(i, j, get_pixel(i, ty, 1));
			}
		}
	}else if(c==0){
		//scroll down
		for(char j = y1; j >= y0; j--){
			for(char i = x1; i >= x0; i--){
				char ty = j-1;
				if(ty <= y0){
					ty = y1;
				}
				sp(i, j, get_pixel(i, ty, 1));
			}
		}
	}else if(c==2){
		//???????
		for (int i = 0; i < (_RESOLUTION_HORZ_BYTES)*_RESOLUTION_VERTICAL; i++){
			g_screen[i] *= 3;
		}
	}
}

void TVout::set_byte(unsigned int i, unsigned char b) {
	g_screen[i] = b;
}

void TVout::fade(char v, char c) {
	if(c==0){
		for (int i = 0; i < (_RESOLUTION_HORZ_BYTES)*_RESOLUTION_VERTICAL; i++){
			g_screen[i] <<= 1;
		}
	}else if(c==1){
		for (int i = 0; i < (_RESOLUTION_HORZ_BYTES)*_RESOLUTION_VERTICAL; i++){
			g_screen[i] >>= 1;
		}
	}else if(c==2){
		for (int i = 0; i < (_RESOLUTION_HORZ_BYTES)*_RESOLUTION_VERTICAL; i++){
			if(g_screen[i] > 0){
				char fb = g_screen[i] >> 4;
				fb *= v*0.099;
				fb <<= 4;
				char lb = g_screen[i] & 15;
				lb *= v*0.099;
				g_screen[i] = lb | fb;
			}
		}
	}else if(c==3){
		unsigned char t = 100;
		for (int i = 0; i < (_RESOLUTION_HORZ_BYTES)*_RESOLUTION_VERTICAL; i++){
			if(g_screen[i] > 0){
				t++;
				if(t > v){
			  		t = 0;
				}
				g_screen[i] = (t*v);
			}
		}	
	}else if(c==4){
		unsigned char t = 0;
		for (int i = 0; i < (_RESOLUTION_HORZ_BYTES)*_RESOLUTION_VERTICAL; i++){
			if(g_screen[i] > 0){
				t++;
				if(t >= v){
					t = 0;
				}
				g_screen[i] ^= (t*v);
			}
		}
	}
}

void TVout::noise(unsigned char v) {
	unsigned int t = 0;
	unsigned char r = v%8; 
	for (int i = 0; i < (_RESOLUTION_HORZ_BYTES)*_RESOLUTION_VERTICAL; i++){
		t++;
		if(t >= v){
			t = 0;
		}
		g_screen[i] ^= (t*v)>>r;
	}
}

/* Inline version of set_pixel that does not perform a bounds check
*/
static void inline sp(unsigned char x, unsigned char y, char c) {
	if (c==1)
	  g_screen[(x/8) + (y*_RESOLUTION_HORZ_BYTES)] |= 0x80 >> (x&7);
    else if (c==0)
	  g_screen[(x/8) + (y*_RESOLUTION_HORZ_BYTES)] &= ~0x80 >> (x&7);
    else
	  g_screen[(x/8) + (y*_RESOLUTION_HORZ_BYTES)] ^= 0x80 >> (x&7);
}

// Inline version of set_pixel that does perform a bounds check


static unsigned char inline gp(unsigned char x, unsigned char y) {
	return ((g_screen[(x/8)+(y*_RESOLUTION_HORZ_BYTES)] >> ((~x) & 7)) &0x01);
}


/* draw a box
 * x1,y1 to x2,y2
 * with color 1 = white, 0=black, 2=invert
 * with fill 0 = black fill, 1 = white fill, 2 = invert fill, 3 = no fill
 * with radius for rounded box
 */
void TVout::draw_box(unsigned char x0, unsigned char y0,
			   unsigned char x1, unsigned char y1, char c, char d, char e)
{
	/* x0,y0 = start position from top left    x1,y1 = width and height
	*  c is box line colour, d is fill colour, e is radius for rounded rectangles (0 radius = straight box)
    *  if the radius is bigger than x0 or y0 then dont render, if the position + length + radius is bigger than the resolution then dont render
    *
    *  It's possible to set a 'render off screen' flag here as well, and use set_pixel(x,y,d) for outside the boundaries and only plot
    *  if it's within the screen. Slower but could allow for more flexibility if desired for partial rounded rectangles. Too many flags for a box draw?
    *
	*  reminder to do some speed tests with set_pixel instead - can we just get away with using that instead?
	*/
    //if (e >= x0_RESOLUTION_HORZ_BYTES || e >= y0 _RESOLUTION_VERTICAL ||
	//	x0 + x1 +e >= _RESOLUTION_HORZ_BYTES || y0 + y1+e >= _RESOLUTION_VERTICAL)
	//  return;
    unsigned char x,y;

    if (e == 0) // ordinary box
    {
	  if (d < 3) // fill the box with the appropriate colour if the fill is 0, 1 or 2. if 3 or higher, no fill
	  {
		for (y = y0+1; y < y0+y1-1; y++)
		{
		    for (x = x0+1; x < x0+x1; x++)
		    {
			  sp(x,y,d);
		    }
		}
	  }
	  // Now draw the box
	  for (y = y0; y < y0+y1; y++)
	  {
		sp(x0,y,c);   // left hand line
		sp(x0+x1,y,c); // right hand line
	  }
	  for (x = x0+1; x < x0 + x1; x++)
	  {
		sp(x,y0,c); // top line
		sp(x,y0+y1-1,c); // bottom line
	  }
    }
    if (e>0)    // rounded rectangle! Split a circle into the 4 corners and draw a box
    {
// subtract the radius of the corners from the straight box we want to draw
	  x0=x0+e;
	  y0=y0+e;
	  y1=y1-e-e;
	  x1=x1-e-e;


// draw the box
	  for (y = y0; y < y0+y1; y++)
	  {
		// First do the left hand line
		sp(x0-e,y+1,c);

		// Now do the right hand line
		sp(x0+x1+e,y+1,c);

	  }

	  for (x = x0+1; x < x0 + x1+1; x++)
	  {
		// First do the top line
		sp(x,y0-e,c);

		// Now do the bottom line
		sp(x,y0+y1+e,c);

	  }



	  int radius = e;
	  int f = 1 - radius;
	  int dx = 1;
	  int dy = -2 * radius;
	  int x = 0;
	  int y = radius;
	  int sty1 = 0;

// draw  quarter circles at each corner of the box

	  f = 1 - radius;
	  dx = 1;
	  dy = -2 * radius;
	  x = 0;
	  y = radius;

	  if (d<3)
	  {
// if filled, use endpoints to draw lines to center x
		for (sty1=0; sty1<radius; sty1++)
		{
		    sp(x0,y0+sty1+y1,d);
		    sp(x0,y0-sty1,d);
		    sp(x0+sty1+x1,y0,d);
		    sp(x0-sty1,y0,d);
		}
	  }
	  sp(x0, y0 + radius+y1,c);
	  sp(x0, y0 - radius,c);
	  sp(x0 + radius+x1, y0,c);
	  sp(x0 - radius, y0,c);
	  while(x < y)
	  {
		if(f >= 0)
		{
		    y--;
		    dy += 2;
		    f += dy;
		}
		x++;
		dx += 2;
		f += dx;
		if (d<3)
		{
// if filled, use endpoints to draw lines to center x
		    for (sty1=0; sty1<y; sty1++)
		    {
			  sp(x0+x+x1,y0+sty1+y1,d);
			  sp(x0-x,y0+sty1+y1,d);
			  sp(x0+x+x1,y0-sty1,d);
			  sp(x0-x,y0-sty1,d);
			  sp(x0+sty1+x1,y0+x+y1,d);
			  sp(x0-sty1,y0+x+y1,d);
			  sp(x0+sty1+x1,y0-x,d);
			  sp(x0-sty1,y0-x,d);
		    }
		}
		sp(x0 + x+x1, y0 + y+y1,c);
		sp(x0 - x, y0 + y+y1,c);
		sp(x0 + x+x1, y0 - y,c);
		sp(x0 - x, y0 - y,c);
		sp(x0 + y+x1, y0 + x+y1,c);
		sp(x0 - y, y0 + x+y1,c);
		sp(x0 + y+x1, y0 - x,c);
		sp(x0 - y, y0 - x,c);
	  }
	  if (d<3)  // redraw circle
	  {
// draw circle
		f = 1 - radius;
		dx = 1;
		dy = -2 * radius;
		x = 0;
		y = radius;

		sp(x0, y0 + radius+y1,c);
		sp(x0, y0 - radius,c);
		sp(x0 + radius+x1, y0,c);
		sp(x0 - radius, y0,c);

		while(x < y)
		{

		    if(f >= 0)
		    {
			  y--;
			  dy += 2;
			  f += dy;
		    }
		    x++;
		    dx += 2;
		    f += dx;
		    sp(x0 + x+x1, y0 + y+y1,c);
		    sp(x0 - x, y0 + y+y1,c);
		    sp(x0 + x+x1, y0 - y,c);
		    sp(x0 - x, y0 - y,c);
		    sp(x0 + y+x1, y0 + x+y1,c);
		    sp(x0 - y, y0 + x+y1,c);
		    sp(x0 + y+x1, y0 - x,c);
		    sp(x0 - y, y0 - x,c);
		}


	  }





    }



}



/* draw a progressbar
 * x1,y1 to x2,y2
 * with color 1 = white, 0=black
 * with 1 = solid, 2 or more = sectioned
 * percentage value for bar, 0 - 100
 */
void TVout::draw_progressbar(unsigned char x0, unsigned char y0,
				     unsigned char x1, unsigned char y1, char c, char d, char e)
{

  //  if (x0 >= _RESOLUTION_HORZ_BYTES || y0 >= _RESOLUTION_VERTICAL ||
//		x0 + x1 >= _RESOLUTION_HORZ_BYTES || y0 + y1 >= _RESOLUTION_VERTICAL || e > 100)
//	  return;
    unsigned char f,x,y,progress;

    if (c > 1)
    {
	  c=1;
    }

    if (c == 1)
    {
	  f=0;
    }

    if (c == 0)
    {
	  f=1;
    }

    //  draw the main box


    for (y = y0; y < y0+y1; y++)
    {
	  // First do the left hand line
	  sp(x0,y,c);

	  // Now do the right hand line
	  sp(x0+x1,y,c);

    }

    for (x = x0+1; x < x0 + x1; x++)
    {
	  // First do the top line
	  sp(x,y0,c);

	  // Now do the bottom line
	  sp(x,y0+y1-1,c);

    }







// now draw an inverted box within


    for (y = y0+1; y < y0+y1-1; y++)
    {
	  // First do the left hand line
	  sp(x0+1,y,f);

	  // Now do the right hand line
	  sp(x0+x1-2,y,f);

    }

    for (x = x0+2; x < x0 + x1-2; x++)
    {
	  // First do the top line
	  sp(x,y0+1,f);

	  // Now do the bottom line
	  sp(x,y0+y1-2,f);

    }


// now draw the progressbar within


    progress = ((x1-4)*e)/100;

    for (x = x0+progress+2+c; x < x0+x1-2; x=x+1)

    {
	  for (y = y0+2; y < y0+y1-2; y++)
	  {

		sp(x,y,f);


	  }
    }

    for (x = x0+2; x < x0+progress+2+c; x=x+d)

    {
	  for (y = y0+2; y < y0+y1-2; y++)
	  {


		sp(x,y,c);

	  }
    }





} 
